pub enum ExitStatus {
Exited(i32),
Signaled(Signal, bool),
}Expand description
Describes the result of a process after it has exited.
This is similar to std::process::ExitStatus, but is easier to match
against and provides additional functionality like raise_or_exit that
helps with propagating an exit status.
Variants
Exited(i32)
Program exited with an exit code.
Signaled(Signal, bool)
Program killed by signal, with or without a coredump.
Implementations
Sourceimpl ExitStatus
impl ExitStatus
Sourcepub fn signal(&self) -> Option<i32>
pub fn signal(&self) -> Option<i32>
If the process was terminated by a signal, returns that signal.
Sourcepub fn success(&self) -> bool
pub fn success(&self) -> bool
Was termination successful? Signal termination is not considered a success, and success is defined as a zero exit status.
Sourcepub fn code(&self) -> Option<i32>
pub fn code(&self) -> Option<i32>
Returns the exit code of the process, if any. If the process was
terminated by a signal, this will return None.
Sourcepub fn raise_or_exit(self) -> !
pub fn raise_or_exit(self) -> !
Propagate the exit status such that the current process exits in the same way that the child process exited.