Filter

Struct Filter 

pub struct Filter { /* private fields */ }
Seccomp-BPF program byte code.

Implementations

Source

impl Filter

pub const fn new() -> Self

Creates a new, empty seccomp program. Note that empty BPF programs are not valid and will fail to load.

pub fn push(&mut self, instruction: sock_filter)

Appends a single instruction to the seccomp-BPF program.

pub fn len(&self) -> usize

Returns the number of instructions in the BPF program.

pub fn is_empty(&self) -> bool

Returns true if the program is empty. Empty seccomp filters will result in an error when loaded.

pub fn instructions(&self) -> &[sock_filter]

Returns the program’s instructions, in order, exactly as they would be loaded.

pub fn load(&self) -> Result<(), Errno>

Loads the program via seccomp into the current process.

Once loaded, the seccomp filter can never be removed. Additional seccomp filters can be loaded, however, and they will chain together and be executed in reverse order.

NOTE: The maximum size of any single seccomp-bpf filter is 4096 instructions. The overall limit is 32768 instructions across all loaded filters.

See seccomp(2) for more details.

pub fn load_and_listen(&self) -> Result<Fd, Errno>

This is the same as Filter::load except that it returns a file descriptor. This is meant to be used with seccomp_unotify(2).