Bind

Struct Bind 

pub struct Bind {
    pub source: CString,
    pub target: CString,
}
Represents a bind mount. Can be converted into a Mount.

Fields

source: CString

The source path of the bind mount. This path must exist. It can be either a file or directory.

target: CString

The target of the bind mount. This does not need to exist and can be created when performing the bind mount.

Implementations

Source

impl Bind

pub fn new<S, T>(source: S, target: T) -> Self
where S: AsRef<OsStr>, T: AsRef<OsStr>,

Creates a new bind mount. The target is optional because it is often convenient to use an identical source and target directory. If target is None, then it is interpretted as being the same as source.