MountFlags

Struct MountFlags 

pub struct MountFlags(/* private fields */);
Used with mount.

Implementations

Source

impl MsFlags

pub const MS_RDONLY: MsFlags

Mount read-only

pub const MS_NOSUID: MsFlags

Ignore suid and sgid bits

pub const MS_NODEV: MsFlags

Disallow access to device special files

pub const MS_NOEXEC: MsFlags

Disallow program execution

pub const MS_SYNCHRONOUS: MsFlags

Writes are synced at once

pub const MS_REMOUNT: MsFlags

Alter flags of a mounted FS

pub const MS_MANDLOCK: MsFlags

Allow mandatory locks on a FS

pub const MS_DIRSYNC: MsFlags

Directory modifications are synchronous

pub const MS_NOATIME: MsFlags

Do not update access times

pub const MS_NODIRATIME: MsFlags

Do not update directory access times

pub const MS_BIND: MsFlags

Linux 2.4.0 - Bind directory at different place

pub const MS_MOVE: MsFlags

Move an existing mount to a new location

pub const MS_REC: MsFlags

Used to create a recursive bind mount.

pub const MS_SILENT: MsFlags

Suppress the display of certain kernel warning messages.

pub const MS_POSIXACL: MsFlags

VFS does not apply the umask

pub const MS_UNBINDABLE: MsFlags

The resulting mount cannot subsequently be bind mounted.

pub const MS_PRIVATE: MsFlags

Make this mount point private.

pub const MS_SLAVE: MsFlags

If this is a shared mount point that is a member of a peer group that contains other members, convert it to a slave mount.

pub const MS_SHARED: MsFlags

Make this mount point shared.

pub const MS_RELATIME: MsFlags

When a file on this filesystem is accessed, update the file’s last access time (atime) only if the current value of atime is less than or equal to the file’s last modification time (mtime) or last status change time (ctime).

pub const MS_KERNMOUNT: MsFlags

👎Deprecated since 0.27.0:

Should only be used in-kernel

Mount request came from within the kernel

pub const MS_I_VERSION: MsFlags

Update inode I_version field

pub const MS_STRICTATIME: MsFlags

Always update the last access time (atime) when files on this filesystem are accessed.

pub const MS_LAZYTIME: MsFlags

Reduce on-disk updates of inode timestamps (atime, mtime, ctime) by maintaining these changes only in memory.

pub const MS_ACTIVE: MsFlags

👎Deprecated since 0.27.0:

Should only be used in-kernel

pub const MS_NOUSER: MsFlags

👎Deprecated since 0.27.0:

Should only be used in-kernel

pub const MS_RMT_MASK: MsFlags

pub const MS_MGC_VAL: MsFlags

pub const MS_MGC_MSK: MsFlags

Source

impl MsFlags

pub const fn empty() -> MsFlags

Get a flags value with all bits unset.

pub const fn all() -> MsFlags

Get a flags value with all known bits set.

pub const fn bits(&self) -> u64

Get the underlying bits value.

The returned value is exactly the bits set in this flags value.

pub const fn from_bits(bits: u64) -> Option<MsFlags>

Convert from a bits value.

This method will return None if any unknown bits are set.

pub const fn from_bits_truncate(bits: u64) -> MsFlags

Convert from a bits value, unsetting any unknown bits.

pub const fn from_bits_retain(bits: u64) -> MsFlags

Convert from a bits value exactly.

pub fn from_name(name: &str) -> Option<MsFlags>

Get a flags value with the bits of a flag with the given name set.

This method will return None if name is empty or doesn’t correspond to any named flag.

pub const fn is_empty(&self) -> bool

Whether all bits in self are unset.

pub const fn is_all(&self) -> bool

Whether all known bits in this flags value are set.

pub const fn intersects(&self, other: MsFlags) -> bool

Whether any set bits in other are also set in self.

pub const fn contains(&self, other: MsFlags) -> bool

Whether all set bits in other are also set in self.

pub fn insert(&mut self, other: MsFlags)

The bitwise or (|) of the bits in self and other.

pub fn remove(&mut self, other: MsFlags)

The intersection of self with the complement of other (&!).

This method is not equivalent to self & !other when other has unknown bits set. remove won’t truncate other, but the ! operator will.

pub fn toggle(&mut self, other: MsFlags)

The bitwise exclusive-or (^) of the bits in self and other.

pub fn set(&mut self, other: MsFlags, value: bool)

Call insert when value is true or remove when value is false.

pub const fn intersection(self, other: MsFlags) -> MsFlags

The bitwise and (&) of the bits in self and other.

pub const fn union(self, other: MsFlags) -> MsFlags

The bitwise or (|) of the bits in self and other.

pub const fn difference(self, other: MsFlags) -> MsFlags

The intersection of self with the complement of other (&!).

This method is not equivalent to self & !other when other has unknown bits set. difference won’t truncate other, but the ! operator will.

pub const fn symmetric_difference(self, other: MsFlags) -> MsFlags

The bitwise exclusive-or (^) of the bits in self and other.

pub const fn complement(self) -> MsFlags

The bitwise negation (!) of the bits in self, truncating the result.

Source

impl MsFlags

pub const fn iter(&self) -> Iter<MsFlags> ⓘ

Yield a set of contained flags values.

Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.

pub const fn iter_names(&self) -> IterNames<MsFlags> ⓘ

Yield a set of contained named flags values.

This method is like iter, except only yields bits in contained named flags. Any unknown bits, or bits not corresponding to a contained flag will not be yielded.