pub struct OwnedDecodeFailure { /* private fields */ }Expand description
Decode refusal retaining the complete bytes and genuine child status.
Implementations
Sourceimpl OwnedDecodeFailure
impl OwnedDecodeFailure
Sourcepub fn cause(&self) -> OwnedRunFailure
pub fn cause(&self) -> OwnedRunFailure
The original typed refusal.
Sourcepub fn encoded_bytes(&self) -> &[u8] ⓘ
pub fn encoded_bytes(&self) -> &[u8] ⓘ
Unchanged bytes which caused the refusal.
Sourcepub fn status(&self) -> ExitStatus
pub fn status(&self) -> ExitStatus
Actual successful child status, which does not erase a decoding failure.