Stdio

Struct Stdio 

pub struct Stdio(/* private fields */);
Describes what to do with a standard I/O stream for a child process when passed to the stdin, stdout, and stderr methods of Command.

Implementations

Source

impl Stdio

pub fn piped() -> Self

A new pipe should be arranged to connect the parent and child processes.

pub fn inherit() -> Self

The child inherits from the corresponding parent descriptor. This is the default mode.

pub fn null() -> Self

This stream will be ignored. This is the equivalent of attaching the stream to /dev/null.