pub struct MountFlags(/* private fields */);Expand description
Used with mount.
Implementations
Sourceimpl MsFlags
impl MsFlags
Sourcepub const MS_SYNCHRONOUS: MsFlags
pub const MS_SYNCHRONOUS: MsFlags
Writes are synced at once
Sourcepub const MS_REMOUNT: MsFlags
pub const MS_REMOUNT: MsFlags
Alter flags of a mounted FS
Sourcepub const MS_MANDLOCK: MsFlags
pub const MS_MANDLOCK: MsFlags
Allow mandatory locks on a FS
Sourcepub const MS_DIRSYNC: MsFlags
pub const MS_DIRSYNC: MsFlags
Directory modifications are synchronous
Sourcepub const MS_NOATIME: MsFlags
pub const MS_NOATIME: MsFlags
Do not update access times
Sourcepub const MS_NODIRATIME: MsFlags
pub const MS_NODIRATIME: MsFlags
Do not update directory access times
Sourcepub const MS_POSIXACL: MsFlags
pub const MS_POSIXACL: MsFlags
VFS does not apply the umask
Sourcepub const MS_UNBINDABLE: MsFlags
pub const MS_UNBINDABLE: MsFlags
The resulting mount cannot subsequently be bind mounted.
Sourcepub const MS_PRIVATE: MsFlags
pub const MS_PRIVATE: MsFlags
Make this mount point private.
Sourcepub const MS_SLAVE: MsFlags
pub const MS_SLAVE: MsFlags
If this is a shared mount point that is a member of a peer group that contains other members, convert it to a slave mount.
Sourcepub const MS_RELATIME: MsFlags
pub const MS_RELATIME: MsFlags
When a file on this filesystem is accessed, update the file’s last access time (atime) only if the current value of atime is less than or equal to the file’s last modification time (mtime) or last status change time (ctime).
Sourcepub const MS_KERNMOUNT: MsFlags
👎Deprecated since 0.27.0: Should only be used in-kernel
pub const MS_KERNMOUNT: MsFlags
Should only be used in-kernel
Mount request came from within the kernel
Sourcepub const MS_I_VERSION: MsFlags
pub const MS_I_VERSION: MsFlags
Update inode I_version field
Sourcepub const MS_STRICTATIME: MsFlags
pub const MS_STRICTATIME: MsFlags
Always update the last access time (atime) when files on this filesystem are accessed.
Sourcepub const MS_LAZYTIME: MsFlags
pub const MS_LAZYTIME: MsFlags
Reduce on-disk updates of inode timestamps (atime, mtime, ctime) by maintaining these changes only in memory.
pub const MS_ACTIVE: MsFlags
Should only be used in-kernel
pub const MS_NOUSER: MsFlags
Should only be used in-kernel
pub const MS_RMT_MASK: MsFlags
pub const MS_MGC_VAL: MsFlags
pub const MS_MGC_MSK: MsFlags
Sourceimpl MsFlags
impl MsFlags
Sourcepub const fn bits(&self) -> u64
pub const fn bits(&self) -> u64
Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Sourcepub const fn from_bits(bits: u64) -> Option<MsFlags>
pub const fn from_bits(bits: u64) -> Option<MsFlags>
Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: u64) -> MsFlags
pub const fn from_bits_truncate(bits: u64) -> MsFlags
Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: u64) -> MsFlags
pub const fn from_bits_retain(bits: u64) -> MsFlags
Convert from a bits value exactly.
Sourcepub fn from_name(name: &str) -> Option<MsFlags>
pub fn from_name(name: &str) -> Option<MsFlags>
Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn intersects(&self, other: MsFlags) -> bool
pub const fn intersects(&self, other: MsFlags) -> bool
Whether any set bits in other are also set in self.
Sourcepub const fn contains(&self, other: MsFlags) -> bool
pub const fn contains(&self, other: MsFlags) -> bool
Whether all set bits in other are also set in self.
Sourcepub fn remove(&mut self, other: MsFlags)
pub fn remove(&mut self, other: MsFlags)
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: MsFlags)
pub fn toggle(&mut self, other: MsFlags)
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub fn set(&mut self, other: MsFlags, value: bool)
pub fn set(&mut self, other: MsFlags, value: bool)
Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(self, other: MsFlags) -> MsFlags
pub const fn intersection(self, other: MsFlags) -> MsFlags
The bitwise and (&) of the bits in self and other.
Sourcepub const fn union(self, other: MsFlags) -> MsFlags
pub const fn union(self, other: MsFlags) -> MsFlags
The bitwise or (|) of the bits in self and other.
Sourcepub const fn difference(self, other: MsFlags) -> MsFlags
pub const fn difference(self, other: MsFlags) -> MsFlags
The intersection of self with the complement of other (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(self, other: MsFlags) -> MsFlags
pub const fn symmetric_difference(self, other: MsFlags) -> MsFlags
The bitwise exclusive-or (^) of the bits in self and other.
Sourcepub const fn complement(self) -> MsFlags
pub const fn complement(self) -> MsFlags
The bitwise negation (!) of the bits in self, truncating the result.